==> af-common-logic-1.1.17.13-sources.jar: com/af/v4/system/common/logic/service/LogicService.java <==
package com.af.v4.system.common.logic.service;

import com.af.v4.system.api.RemoteLogicService;
import com.af.v4.system.api.factory.DynamicFeignClientFactory;
import com.af.v4.system.common.core.constant.CodeNormsConstants;
import com.af.v4.system.common.core.constant.SecurityConstants;
import com.af.v4.system.common.core.domain.R;
import com.af.v4.system.common.core.exception.LogicException;
import com.af.v4.system.common.core.exception.ServiceException;
import com.af.v4.system.common.datasource.DynamicDataSource;
import com.af.v4.system.common.expression.Expression;
import com.af.v4.system.common.liuli.utils.ApplicationUtils;
import com.af.v4.system.common.resource.mapper.LogicMapper;
import com.af.v4.system.common.resource.mapper.PluginMapper;
import org.apache.logging.log4j.LogManager;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.function.Supplier;

@Service
@Transactional(rollbackFor = Exception.class)
public class LogicService {
    private static final Logger LOGGER = LoggerFactory.getLogger(LogicService.class);
    private static final org.apache.logging.log4j.Logger LOG4J_LOGGER = LogManager.getLogger(LogicService.class);
    private final LogicNewTranService logicNewTranService;
    private final DynamicFeignClientFactory<RemoteLogicService> dynamicFeignClientFactory;
    private final LogicMapper logicMapper;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final ApplicationUtils applicationUtils;

    public LogicService(LogicNewTranService newLogicService, DynamicFeignClientFactory<RemoteLogicService> dynamicFeignClientFactory, LogicMapper logicMapper, @Qualifier("logicAsyncTaskExecutor") ThreadPoolTaskExecutor taskExecutor, ApplicationUtils applicationUtils) {
        this.logicNewTranService = newLogicService;
        this.dynamicFeignClientFactory = dynamicFeignClientFactory;
        this.logicMapper = logicMapper;
        this.taskExecutor = taskExecutor;
        this.applicationUtils = applicationUtils;
    }

    public static void setDataSource(String dataSourceName) {
        DynamicDataSource.setDataSource(dataSourceName);
    }

    public static void pollDataSource() {
        DynamicDataSource.pollDataSource();
    }

    /**
     * 获取logic使用的数据源名称
     * 优先级从高到低排列如下
     * 1. run方法中指定的数据源名称
     * 2. logic注册文件中指定的数据源名称
     * 3. 当前线程中已有的数据源名称
     *
     * @param dataSourceName 指定的数据源名称
     * @param resource       logic注册资源
     * @return 数据源名称
     */
    private static String getDataSourceName(String dataSourceName, LogicMapper.LogicResource resource) {
        String dataSource;
        if (dataSourceName == null) {
            String mappingDataSource = resource.getDataSource();
            if (mappingDataSource != null) {
                dataSource = mappingDataSource;
            } else {
                dataSource = DynamicDataSource.getDataSource();
            }
        } else {
            dataSource = dataSourceName;
        }
        return dataSource;
    }

    /**
     * 是否存在指定名称的Logic资源
     *
     * @param logicName Logic名称
     * @return 是否存在
     */
    public boolean has(String logicName) {
        return logicMapper.hasResource(logicName);
    }

    /**
     * 执行Logic
     *
     * @param logicName      Logic名称
     * @param param          参数
     * @param dataSourceName 数据源名称
     * @return 执行结果
     */
    public Object run(String logicName, JSONObject param, String dataSourceName) {
        // 获取Logic资源
        LogicMapper.LogicResource resource = logicMapper.getResource(logicName);
        String name = resource.getAlias();
        boolean cache = resource.isCache();
        String dataSource = getDataSourceName(dataSourceName, resource);
        LOGGER.info(CodeNormsConstants.DEBUG_PREFIX + "执行Logic[{}]，useCache: {}，params: {}，dataSource：{}", name, cache, param, dataSource);
        if (resource.isDeprecated()) {
            String replacedBy = resource.getReplacedBy();
            if (replacedBy != null) {
                LOGGER.warn("Logic[{}]已标记废弃，可能会在后续版本中移除，请迁移到新API[{}]", name, replacedBy);
            } else {
                LOGGER.warn("Logic[{}]已标记废弃，可能会在后续版本中移除", name);
            }
        }
        String source = resource.getSource();
        // 附加用户注册的对象到业务逻辑中
        Map<String, Object> params = new HashMap<>(PluginMapper.getPluginMap());
        params.put("data", param);
        params.put("log", LOG4J_LOGGER);
        params.put("ENV", applicationUtils.getValues());
        params.put("RESOURCE", resource.getJsonParams());
        params.put("logic", this);

        Supplier<Object> action = () -> logicMonitor(resource.getAlias(), resource.getPath().toString(), dataSource, param.toString(), () -> runExpression(source, params));
        return action.get();
    }

    public Object run(String name, JSONObject param) {
        return this.run(name, param, null);
    }

    public Object run(String name, String str) {
        return this.run(name, buildLogicParams(str));
    }

    /**
     * 异步执行Logic
     *
     * @param name  Logic名称
     * @param param 参数
     */
    public CompletableFuture<Object> runAsync(String name, JSONObject param) {
        return CompletableFuture.supplyAsync(() -> this.run(name, param), taskExecutor).exceptionally(throwable -> {
            LOGGER.error("异步请求发生异常.", throwable);
            return null;
        });
    }

    public CompletableFuture<Object> runAsync(String name, String str) {
        return this.runAsync(name, buildLogicParams(str));
    }

    /**
     * 远程调用Logic
     *
     * @param serviceName 服务名
     * @param logicName   Logic名称
     * @param params      Logic参数
     * @return 执行结果
     */
    //    @GlobalTransactional
    public JSONObject remoteRun(String serviceName, String logicName, String params) {
        LOGGER.info(CodeNormsConstants.DEBUG_PREFIX + "远程调用{}的Logic[{}]，参数：{}", serviceName, logicName, params);
        return (JSONObject) logicMonitor(logicName, "http://" + serviceName + "/logic/" + logicName, null, params, () -> {
            RemoteLogicService remoteLogicService = dynamicFeignClientFactory.getFeignClient(RemoteLogicService.class, serviceName);
            R<Object> logicResult = remoteLogicService.run(logicName, params, SecurityConstants.INNER);
            return logicResult.parseResponseJson();
        });
    }

    //    @GlobalTransactional
    public JSONObject remoteRun(String serviceName, String logicName, JSONObject params) {
        return remoteRun(serviceName, logicName, params.toString());
    }

    /**
     * 执行原生表达式
     *
     * @param source 表达式内容
     * @param params 参数
     * @return 执行结果
     */
    public Object runExpression(String source, Map<String, Object> params) {
        return Expression.run(source, params);
    }

    /**
     * 构建业务异常
     *
     * @param ex 运行时异常
     * @return 业务异常
     */
    private ServiceException buildServiceException(String logicName, String logicPath, String logicParams, Exception ex) {
        Throwable cause = ex.getCause();
        if (cause instanceof InvocationTargetException targetException) {
            cause = targetException.getTargetException();
        }
        if (cause instanceof LogicException logicException) {
            return new LogicException(logicException.getMessage(), logicException.getCode(), ex, logicName, logicParams, logicPath, logicException);
        } else if (cause instanceof ServiceException serviceException) {
            return new LogicException(serviceException.getMessage(), serviceException.getCode(), ex, logicName, logicParams, logicPath);
        } else {
            return new LogicException(ex, logicName, logicParams, logicPath);
        }
    }

    /**
     * 构建Logic参数
     *
     * @param str 字符串参数
     * @return JSON格式参数
     */
    private JSONObject buildLogicParams(String str) {
        JSONObject param = null;
        // 把传递过来的参数，放到data里，以便跟entity，sql等对象区别开来
        // 判断传来的数据是否是XML格式
        str = str.trim();
        if (str.startsWith("<") && str.endsWith(">")) {
            // 传来数据为XML格式
            param = new JSONObject();
            param.put("xml", str);
        } else {
            // 传来数据为JSON格式
            Object json = new JSONTokener(str).nextValue();
            if (json instanceof JSONObject st) {
                param = st;
                if (!param.isNull("data")) {
                    Object dataParam = param.get("data");
                    if (dataParam instanceof JSONObject) {
                        param = (JSONObject) dataParam;
                        param.put("standardData", new JSONObject(str).toString());
                    }
                }
            } else if (json instanceof JSONArray ar) {
                param = new JSONObject();
                param.put("arr", ar);
            }
        }
        return param;
    }

    private Object logicMonitor(String name, String path, String dataSource, String params, Supplier<Object> business, Supplier<Object> finallyFun) {
        long begin = System.currentTimeMillis();
        try {
            return DynamicDataSource.withDataSource(dataSource, business);
        } catch (Exception e) {
            throw buildServiceException(name, path, params, e);
        } finally {
            if (finallyFun != null) {
                finallyFun.get();
            }
            long end = System.currentTimeMillis();
            long time = end - begin;
            String engMessage = CodeNormsConstants.DEBUG_PREFIX + "Logic[" + name + "]处理耗时:" + time + "ms";
            if (time >= 8000) {
                LOGGER.error(engMessage);
            } else if (time >= 4000) {
                LOGGER.warn(engMessage);
            } else {
                LOGGER.info(engMessage);
            }
        }
    }

    private Object logicMonitor(String name, String path, String dataSource, String params, Supplier<Object> business) {
        return logicMonitor(name, path, dataSource, params, business, null);
    }
}

==> af-common-logic-1.1.17.27-sources.jar: com/af/v4/system/common/logic/service/LogicService.java <==
package com.af.v4.system.common.logic.service;

import com.af.v4.system.api.RemoteLogicService;
import com.af.v4.system.api.factory.DynamicFeignClientFactory;
import com.af.v4.system.common.core.constant.CodeNormsConstants;
import com.af.v4.system.common.core.constant.SecurityConstants;
import com.af.v4.system.common.core.domain.R;
import com.af.v4.system.common.core.exception.LogicException;
import com.af.v4.system.common.core.exception.ServiceException;
import com.af.v4.system.common.datasource.DynamicDataSource;
import com.af.v4.system.common.expression.Expression;
import com.af.v4.system.common.liuli.utils.ApplicationUtils;
import com.af.v4.system.common.redis.RedisService;
import com.af.v4.system.common.resource.constant.CacheKey;
import com.af.v4.system.common.resource.mapper.LogicMapper;
import com.af.v4.system.common.resource.mapper.PluginMapper;
import org.apache.logging.log4j.LogManager;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.function.Supplier;

@Service
@Transactional(rollbackFor = Exception.class)
public class LogicService {
    private static final Logger LOGGER = LoggerFactory.getLogger(LogicService.class);
    private static final org.apache.logging.log4j.Logger LOG4J_LOGGER = LogManager.getLogger(LogicService.class);
    private final LogicNewTranService logicNewTranService;
    private final DynamicFeignClientFactory<RemoteLogicService> dynamicFeignClientFactory;
    private final LogicMapper logicMapper;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final ApplicationUtils applicationUtils;
    private final RedisService redisService;

    public LogicService(LogicNewTranService newLogicService, DynamicFeignClientFactory<RemoteLogicService> dynamicFeignClientFactory, LogicMapper logicMapper, @Qualifier("logicAsyncTaskExecutor") ThreadPoolTaskExecutor taskExecutor, ApplicationUtils applicationUtils, RedisService redisService) {
        this.logicNewTranService = newLogicService;
        this.dynamicFeignClientFactory = dynamicFeignClientFactory;
        this.logicMapper = logicMapper;
        this.taskExecutor = taskExecutor;
        this.applicationUtils = applicationUtils;
        this.redisService = redisService;
    }

    public static void setDataSource(String dataSourceName) {
        DynamicDataSource.setDataSource(dataSourceName);
    }

    public static void pollDataSource() {
        DynamicDataSource.pollDataSource();
    }

    /**
     * 获取logic使用的数据源名称
     * 优先级从高到低排列如下
     * 1. run方法中指定的数据源名称
     * 2. logic注册文件中指定的数据源名称
     * 3. 当前线程中已有的数据源名称
     *
     * @param dataSourceName 指定的数据源名称
     * @param resource       logic注册资源
     * @return 数据源名称
     */
    private static String getDataSourceName(String dataSourceName, LogicMapper.LogicResource resource) {
        String dataSource;
        if (dataSourceName == null) {
            String mappingDataSource = resource.getDataSource();
            if (mappingDataSource != null) {
                dataSource = mappingDataSource;
            } else {
                dataSource = DynamicDataSource.getDataSource();
            }
        } else {
            dataSource = dataSourceName;
        }
        return dataSource;
    }

    /**
     * 是否存在指定名称的Logic资源
     *
     * @param logicName Logic名称
     * @return 是否存在
     */
    public boolean has(String logicName) {
        return logicMapper.hasResource(logicName);
    }

    /**
     * 执行Logic
     *
     * @param logicName      Logic名称
     * @param param          参数
     * @param dataSourceName 数据源名称
     * @return 执行结果
     */
    public Object run(String logicName, JSONObject param, String dataSourceName) {
        // 获取Logic资源
        LogicMapper.LogicResource resource = logicMapper.getResource(logicName);
        String name = resource.getAlias();
        boolean cache = resource.isCache();
        String dataSource = getDataSourceName(dataSourceName, resource);
        LOGGER.info(CodeNormsConstants.DEBUG_PREFIX + "执行Logic[{}]，useCache: {}，params: {}，dataSource：{}", name, cache, param, dataSource);
        if (resource.isDeprecated()) {
            String replacedBy = resource.getReplacedBy();
            if (replacedBy != null) {
                LOGGER.warn("Logic[{}]已标记废弃，可能会在后续版本中移除，请迁移到新API[{}]", name, replacedBy);
            } else {
                LOGGER.warn("Logic[{}]已标记废弃，可能会在后续版本中移除", name);
            }
        }
        String source = resource.getSource();
        // 附加用户注册的对象到业务逻辑中
        Map<String, Object> params = new HashMap<>(PluginMapper.getPluginMap());
        params.put("data", param);
        params.put("log", LOG4J_LOGGER);
        params.put("ENV", applicationUtils.getValues());
        params.put("RESOURCE", resource.getJsonParams());
        params.put("logic", this);

        Supplier<Object> action = () -> logicMonitor(resource.getAlias(), resource.getPath(), dataSource, param.toString(), () -> runExpression(source, params));
        return action.get();
    }

    public Object run(String name, JSONObject param) {
        return this.run(name, param, null);
    }

    public Object run(String name, String str) {
        return this.run(name, buildLogicParams(str));
    }

    /**
     * 异步执行Logic
     *
     * @param name  Logic名称
     * @param param 参数
     */
    public CompletableFuture<Object> runAsync(String name, JSONObject param) {
        return CompletableFuture.supplyAsync(() -> this.run(name, param), taskExecutor).exceptionally(throwable -> {
            LOGGER.error("异步请求发生异常.", throwable);
            return null;
        });
    }

    public CompletableFuture<Object> runAsync(String name, String str) {
        return this.runAsync(name, buildLogicParams(str));
    }

    /**
     * 远程调用Logic
     *
     * @param serviceName 服务名
     * @param logicName   Logic名称
     * @param params      Logic参数
     * @return 执行结果
     */
    //    @GlobalTransactional
    public JSONObject remoteRun(String serviceName, String logicName, String params) {
        LOGGER.info(CodeNormsConstants.DEBUG_PREFIX + "远程调用{}的Logic[{}]，参数：{}", serviceName, logicName, params);
        return (JSONObject) logicMonitor(logicName, "http://" + serviceName + "/logic/" + logicName, null, params, () -> {
            RemoteLogicService remoteLogicService = dynamicFeignClientFactory.getFeignClient(RemoteLogicService.class, serviceName);
            R<Object> logicResult = remoteLogicService.run(logicName, params, SecurityConstants.INNER);
            return logicResult.parseResponseJson();
        });
    }

    //    @GlobalTransactional
    public JSONObject remoteRun(String serviceName, String logicName, JSONObject params) {
        return remoteRun(serviceName, logicName, params.toString());
    }

    /**
     * 执行原生表达式
     *
     * @param source 表达式内容
     * @param params 参数
     * @return 执行结果
     */
    public Object runExpression(String source, Map<String, Object> params) {
        return Expression.run(source, params);
    }

    /**
     * 构建业务异常
     *
     * @param ex 运行时异常
     * @return 业务异常
     */
    private ServiceException buildServiceException(String logicName, String logicPath, String logicParams, Exception ex) {
        Throwable cause = ex.getCause();
        if (cause instanceof InvocationTargetException targetException) {
            cause = targetException.getTargetException();
        }
        if (cause instanceof LogicException logicException) {
            return new LogicException(logicException.getMessage(), logicException.getCode(), ex, logicName, logicParams, logicPath, logicException);
        } else if (cause instanceof ServiceException serviceException) {
            return new LogicException(serviceException.getMessage(), serviceException.getCode(), ex, logicName, logicParams, logicPath);
        } else {
            return new LogicException(ex, logicName, logicParams, logicPath);
        }
    }

    /**
     * 构建Logic参数
     *
     * @param str 字符串参数
     * @return JSON格式参数
     */
    private JSONObject buildLogicParams(String str) {
        JSONObject param = null;
        // 把传递过来的参数，放到data里，以便跟entity，sql等对象区别开来
        // 判断传来的数据是否是XML格式
        str = str.trim();
        if (str.startsWith("<") && str.endsWith(">")) {
            // 传来数据为XML格式
            param = new JSONObject();
            param.put("xml", str);
        } else {
            // 传来数据为JSON格式
            Object json = new JSONTokener(str).nextValue();
            if (json instanceof JSONObject st) {
                param = st;
                if (!param.isNull("data")) {
                    Object dataParam = param.get("data");
                    if (dataParam instanceof JSONObject) {
                        param = (JSONObject) dataParam;
                        param.put("standardData", new JSONObject(str).toString());
                    }
                }
            } else if (json instanceof JSONArray ar) {
                param = new JSONObject();
                param.put("arr", ar);
            }
        }
        return param;
    }

    private Object logicMonitor(String name, String path, String dataSource, String params, Supplier<Object> business, Supplier<Object> finallyFun) {
        long begin = System.currentTimeMillis();
        try {
            return DynamicDataSource.withDataSource(dataSource, business);
        } catch (Exception e) {
            throw buildServiceException(name, path, params, e);
        } finally {
            if (finallyFun != null) {
                finallyFun.get();
            }
            long end = System.currentTimeMillis();
            long time = end - begin;
            String engMessage = CodeNormsConstants.DEBUG_PREFIX + "Logic[" + name + "]处理耗时:" + time + "ms";
            if (time >= 8000) {
                LOGGER.error(engMessage);
            } else if (time >= 4000) {
                LOGGER.warn(engMessage);
            } else {
                LOGGER.info(engMessage);
            }
        }
    }

    private Object logicMonitor(String name, String path, String dataSource, String params, Supplier<Object> business) {
        return logicMonitor(name, path, dataSource, params, business, null);
    }

    public void deleteCache(String logicName) {
        String cacheKey = CacheKey.getLogicDataCacheKey(logicName, "*");
        Iterable<String> cacheKeyList = redisService.getKeys(cacheKey);
        redisService.deleteList(cacheKeyList);
        LOGGER.info("手动清除Logic[{}]缓存完成", logicName);
    }
}
